import Template from './Template.js';
import DateUtil from '@midefos/utils/src/DateUtil.js'

export default class TableColumns {

    static _defaultConfig() {
        return {
            maxWidth: 200,
            tooltip: true,
            resizable: false
        }
    }

    static _defaultTextConfig() {
        const config = TableColumns._defaultConfig();
        const textConfig = { headerFilter: true };
        return { ...config, ...textConfig };
    }

    static hidden(key) {
        const config = TableColumns._defaultConfig();
        const hiddenConfig = { field: key, visible: false };
        return { ...config, ...hiddenConfig };
    }

    static defaultBoolean(key, text) {
        const config = TableColumns._defaultConfig();
        const defaultBooleanConfig = { field: key, title: text, formatter: "tickCross", sorter: "boolean", hozAlign: "center" };
        return { ...config, ...defaultBooleanConfig };
    }

    static text(key, text) {
        const config = TableColumns._defaultTextConfig();
        const textConfig = { title: text, field: key };
        return { ...config, ...textConfig };
    }

    static type(key, text) {
        const textColumn = TableColumns.text(key, text);
        const typeConfig = { headerFilter: 'select', headerFilterParams: { values: true } };
        return { ...textColumn, ...typeConfig };
    }

    static internal_link(textKey, URL, text) {
        const config = TableColumns._defaultTextConfig();
        const internalLinkConfig = {
            title: text, field: textKey,
            formatter: "link", formatterParams: { url: URL }
        }
        return { ...config, ...internalLinkConfig };
    }

    static link(textKey, URLKey, text) {
        const config = TableColumns._defaultTextConfig();
        const linkConfig = {
            title: text, field: textKey,
            formatter: "link", formatterParams: { urlField: URLKey, target: "_" }
        }
        return { ...config, ...linkConfig };
    }

    static image(key) {
        const config = TableColumns._defaultConfig();

        const width = 75;
        const height = 75;
        const imageConfig = {
            field: key, headerSort: false,
            formatter: (cell) => {
                let image = cell.getValue();
                if (!image) image = 'logo.png';

                return `<div class="thumbnailImageContainer"
                style="height: ${height}px; width: ${width}px;">
                <img style="height: ${height}px; width: ${width}px;" src='${image}' />
                </div>
                <div class="hoverImageContainer">
                <img style="bottom: calc(${height}px + 10px); left: calc(${width}px + 10px);"
                src='${image}' />
                </div>`;
            }
        }
        return { ...config, ...imageConfig };
    }

    static price(key, text = 'Price') {
        const config = TableColumns._defaultTextConfig();
        const priceConfig = {
            title: text, field: key,
            formatter: (cell) => {
                const item = cell.getRow().getData();
                return TableColumns._buildPrice(item, key);
            }
        }
        return { ...config, ...priceConfig };
    }

    static _buildPrice(item, key) {
        let text = `<div>${item[key]}€`;
        text += '</div>';
        return text;
    }

    static _calculateFee(price, fee) {
        price *= fee;
        return Math.round(price * 100) / 100;
    }

    static datetime(key, text) {
        const config = TableColumns._defaultTextConfig();
        const datetimeConfig = {
            field: key, title: text,
            formatter: (cell) => {
                const value = cell.getValue();
                if (!value) return null;
                return new Date(value).toLocaleString();
            }
        }
        return { ...config, ...datetimeConfig };
    }

    static date(key, text) {
        const config = TableColumns._defaultTextConfig();
        const dateConfig = {
            field: key, title: text,
            formatter: (cell) => {
                const value = cell.getValue();
                if (!value) return null;
                return new Date(value).toLocaleDateString();
            }
        }
        return { ...config, ...dateConfig };
    }

    static time(key, text) {
        const config = TableColumns._defaultTextConfig();
        const timeConfig = {
            field: key, title: text,
            formatter: (cell) => {
                const value = cell.getValue();
                if (!value) return null;
                return new Date(value).toLocaleTimeString();
            }
        }
        return { ...config, ...timeConfig };
    }

    static dateDiff(startKey, endKey, text) {
        const config = TableColumns._defaultTextConfig();
        const dateDiffConfig = {
            title: text,
            formatter: (cell) => {
                const item = cell.getRow().getData();
                return DateUtil.millisecondsToDHMS(DateUtil.difference(new Date(item[endKey]), new Date(item[startKey])));
            }
        }
        return { ...config, ...dateDiffConfig };
    }

    static quantity(key, text, formater = null, sorter = null) {
        const config = TableColumns._defaultTextConfig();
        if (!formater) {
            formater = (cell) => {
                const item = cell.getRow().getData();
                const items = item[key];
                return items?.length || 0;
            }
            sorter =  (a, b) => a.length - b.length;
        }
        const quantityConfig = {
            field: key, title: text, 
            formatter: formater, sorter: sorter
        }
        return { ...config, ...quantityConfig };
    }

    static remove(key, action) {
        const config = TableColumns._defaultConfig();
        const removeConfig = {
            field: key, headerSort: false,
            formatter: () => Template.trashIcon(),
            cellClick: async (event, cell) => {
                await action(cell.getRow().getData());
            }
        }
        return { ...config, ...removeConfig };
    }

}